#import "BackupViewController.h"
#import "ASIHTTPRequest.h"
#import "ASINetworkQueue.h"
#import "SBJson.h"
#import "BackpackAppDelegate.h"
#import "InstagramDataParser.h"
#import "EMKeychainItem.h"

@implementation BackupViewController

@synthesize startDownloadView, showProgressView, downloadProgressMessage, finishedDownloadView;
@synthesize usernameField, downloadProgressIndicator, userPhotoCount, userFollowsCount, userFollowerCount;
@synthesize instaParser;
@synthesize networkQueue, saveFolderPath;


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
    }
    return self;
}

- (void)loadView {
    [super loadView];
    [self showUserDetails];
}

- (IBAction)backupClicked:(id)sender {
    
    openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseFiles:NO];
    [openPanel setCanChooseDirectories:YES];
    [openPanel setAllowsMultipleSelection:NO];
    [openPanel setCanCreateDirectories:YES];
    [openPanel setPrompt:@"Select"];
    [openPanel setTitle:@"Choose location for your photos"];
    
    NSInteger result;
    result = [openPanel runModal];
    if (result == NSOKButton) {
        NSString* userSelectedPath = [[openPanel URL] relativePath];
        [self setSaveFolderPath:userSelectedPath];
        // NSLog(@"Saving photos to: %@", saveFolderPath);
    } else {
        return;
    }
    
    // write the initial text for progress, making sure it's from 0 always
    NSString *message = @"starting...";
    [downloadProgressMessage setStringValue:message];
    
    // progressbar itself
    [[self downloadProgressIndicator] setDoubleValue:0];
    
    [[self.view animator] replaceSubview:startDownloadView with:showProgressView];
    
    // start sequence to download data and make backup..
    [self backupPhotos];
}

- (IBAction)cancelBackup:(id)sender {
    // cancel and clean
    [[self networkQueue] cancelAllOperations];
    [self setSaveFolderPath:nil];
    [[self.view animator] replaceSubview:showProgressView with:startDownloadView];
}

- (IBAction)selectLocation:(id)sender {
    
    openPanel = [NSOpenPanel openPanel];
    [openPanel setCanChooseFiles:NO];
    [openPanel setCanChooseDirectories:YES];
    [openPanel setAllowsMultipleSelection:NO];
    [openPanel setCanCreateDirectories:YES];
    
    NSInteger result;
    result = [openPanel runModal];
    if (result == NSOKButton) {
        NSString* userSelectedPath = [[openPanel URL] relativePath];
        [self setSaveFolderPath:userSelectedPath];
        // NSLog(@"Saving photos to: %@", saveFolderPath);
        
        NSArray *myArray = [userSelectedPath componentsSeparatedByString: @"/"];
        NSString *fileName = (NSString*)[myArray lastObject];
        NSLog(@"last element is: %@", fileName);
    }
}

- (void) showUserDetails {
    
    BackpackAppDelegate *appDelegate = (BackpackAppDelegate *)[[NSApplication sharedApplication] delegate];
    NSString *tempToken = [appDelegate tempUserToken];
    
    instaParser = [[InstagramDataParser alloc] init];
    [[self instaParser] setDelegate:self];
    [instaParser setToken:tempToken];
    [instaParser setDidFinishUserInfo:@selector(userInfoLoaded)];
    [instaParser getUserInfo];
    
    [instaParser authenticateUser];
}
     
- (void) userInfoLoaded {
    // NSLog(@"success with userInfo");
    NSString *vPhotoCount = [[instaParser photoCount] stringValue];
    NSString *vFollowsCount = [[instaParser followsCount] stringValue];
    NSString *vFollowerCount = [[instaParser followerCount] stringValue];
    
    [usernameField setStringValue: [instaParser username]];
    [userPhotoCount setStringValue:vPhotoCount];
    [userFollowsCount setStringValue:vFollowsCount];
    [userFollowerCount setStringValue:vFollowerCount];
    
    NSString *message = [NSString stringWithFormat:@"%@", vPhotoCount]; 
    [messageField setStringValue:message];
    
}

- (void) backupPhotos {
    
    [instaParser setDidFinishGathering:@selector(photoListGathered)];
    [instaParser gatherData];
    
}

-(void)photoListGathered {
    // NSLog(@"ViewController got it, thanks");
    NSArray *photolist = [instaParser getPhotolist];
    
    // WE HAVE PHOTOS, LETS MAKE DIRECTORY FOR THEM BEFORE SAVING STATS
    
    if (![self saveFolderPath]) {
        NSArray *paths = NSSearchPathForDirectoriesInDomains (NSDesktopDirectory, NSUserDomainMask, YES);
        NSString *basepath = [paths objectAtIndex:0];
        NSString *defaultFolderName = [NSString stringWithFormat: @"Instagram Photos - %@", [instaParser username]];
        
        [self setSaveFolderPath:[basepath stringByAppendingPathComponent:defaultFolderName]];
        
        // MAKE SURE DEFAULT PATH EXISTS
        NSFileManager *fileManager= [NSFileManager defaultManager]; 
        if(![fileManager fileExistsAtPath:[self saveFolderPath]])
            if(![fileManager createDirectoryAtPath:[self saveFolderPath] withIntermediateDirectories:YES attributes:nil error:NULL])
                NSLog(@"Error: Create folder failed %@", [self saveFolderPath]);
    }
    
    // NSLog(@"Filepath for photos: %@", saveFolderPath);
    // [self setSaveFolderPath:saveFolderPath]; // make it into allocated variable, used later
    
	[[self networkQueue] cancelAllOperations];
    
	// Creating a new queue each time we use it means we don't have to worry about clearing delegates or resetting progress tracking
	[self setNetworkQueue:[ASINetworkQueue queue]];
	[[self networkQueue] setDelegate:self];
	[[self networkQueue] setRequestDidFinishSelector:@selector(requestFinished:)];
	[[self networkQueue] setRequestDidFailSelector:@selector(requestFailed:)];
	[[self networkQueue] setQueueDidFinishSelector:@selector(queueFinished:)];
    
    // SETUP PROGRESS BAR
    NSInteger itemCount = [photolist count];
    [downloadProgressIndicator setMaxValue: itemCount];
    
	int i;
	for (i=0; i<itemCount; i++) {
        NSDictionary *newP = [photolist objectAtIndex:i];
        NSString *path = [[newP objectForKey:@"standard_resolution"] valueForKey:@"url"];
        // NSLog(@"path is = %@", path);
        
		ASIHTTPRequest *request = [ASIHTTPRequest requestWithURL:[NSURL URLWithString:path]];
        [request setDownloadDestinationPath:[NSString stringWithFormat:@"%@/%@-%d.jpg", saveFolderPath,[instaParser username], (i+1)]];
		[[self networkQueue] addOperation:request];
	}
    
    
	[[self networkQueue] go];
}

- (void)requestFinished:(ASIHTTPRequest *)request
{
	// You could release the queue here if you wanted
	if ([[self networkQueue] requestsCount] == 0) {
        
		// Since this is a retained property, setting it to nil will release it
		// This is the safest way to handle releasing things - most of the time you only ever need to release in your accessors
		// And if you an Objective-C 2.0 property for the queue (as in this example) the accessor is generated automatically for you
		[self setNetworkQueue:nil]; 
	}
    
    // show progress bar increment
    [downloadProgressIndicator incrementBy:1];
    NSInteger numDownloaded = [downloadProgressIndicator doubleValue];
    NSInteger numAll = [downloadProgressIndicator maxValue];
    NSString *dlProgress = [NSString stringWithFormat:@"%ld / %ld", numDownloaded, numAll];
    [downloadProgressMessage setStringValue:dlProgress];
    
	//... Handle success
	// NSLog(@"Request finished");
}

- (void)requestFailed:(ASIHTTPRequest *)request
{
	// You could release the queue here if you wanted
	if ([[self networkQueue] requestsCount] == 0) {
		[self setNetworkQueue:nil]; 
	}
    
	//... Handle failure
	// NSLog(@"Request failed");
}


- (void)queueFinished:(ASINetworkQueue *)queue
{
	// You could release the queue here if you wanted
	if ([[self networkQueue] requestsCount] == 0) {
		[self setNetworkQueue:nil]; 
	}
	// NSLog(@"Queue finished");
    
    // SHOW NEW VIEW INSIDE
    // [[self view] replaceSubview:showProgressView with:startDownloadView];
    [[self.view animator] replaceSubview:showProgressView with:finishedDownloadView];
    // and lets inform interface we are done loading
    // BackpackAppDelegate *appDelegate = (BackpackAppDelegate *)[[NSApplication sharedApplication] delegate];
    // [appDelegate closeBackupView];
}

// AFTER SCREENS
- (IBAction)showInFinder:(id)sender {
    // NSLog(@"fileURL is %@", saveFolderPath);
    NSURL *fileURL = [NSURL fileURLWithPath:saveFolderPath];
    NSArray *fileURLs = [NSArray arrayWithObjects:fileURL, nil];
    [[NSWorkspace sharedWorkspace] activateFileViewerSelectingURLs:fileURLs];
}

- (IBAction)startAgain:(id)sender {
    // TODO: RESET SOME PARAMS HERE TOO?
    [[self.view animator] replaceSubview:finishedDownloadView with:startDownloadView];
}


- (void)dealloc
{
    [instaParser release];
    [networkQueue release];
    [super dealloc];
}

@end
